@@ -518,11 +518,11 @@ krb5_boolean find_first_cmd_that_exists(fcmd_arr, cmd_out, err_out)
         for(j= 0; j < i; j ++)
             k5_buf_add_fmt(&buf, " %s ", fcmd_arr[j]);
         k5_buf_add(&buf, "\n");
-        *err_out = k5_buf_data(&buf);
-        if (*err_out == NULL) {
+        if (k5_buf_status(&buf) != 0) {
             perror(prog_name);
             exit(1);
         }
+        *err_out = buf.data;
     }
 
 
