@@ -90,7 +90,6 @@ extern int daemon(int, int);
 
 #define SYSLOG_CLASS LOG_DAEMON
 
-char *def_realm = NULL;
 int runonce = 0;
 
 /*
@@ -128,6 +127,7 @@ static krb5_principal client;   /* This is who we're talking to */
 static krb5_context kpropd_context;
 static krb5_auth_context auth_context;
 static char *realm = NULL;      /* Our realm */
+static char *def_realm = NULL;  /* Ref pointer for default realm */
 static char *file = KPROPD_DEFAULT_FILE;
 static char *temp_file_name;
 static char *kdb5_util = KPROPD_DEFAULT_KDB5_UTIL;
@@ -631,53 +631,26 @@ do_iprop()
     if (pollin == 0)
         pollin = 10;
 
-    /* Grab the realm info and check if iprop is enabled. */
-    if (def_realm == NULL) {
-        retval = krb5_get_default_realm(kpropd_context, &def_realm);
-        if (retval) {
-            com_err(progname, retval, _("Unable to get default realm"));
-            return retval;
-        }
-    }
-
-    params.mask |= KADM5_CONFIG_REALM;
-    params.realm = def_realm;
-
     if (master_svc_princstr == NULL) {
-        retval = kadm5_get_kiprop_host_srv_name(kpropd_context, def_realm,
+        retval = kadm5_get_kiprop_host_srv_name(kpropd_context, realm,
                                                 &master_svc_princstr);
         if (retval) {
             com_err(progname, retval,
                     _("%s: unable to get kiprop host based "
                       "service name for realm %s\n"),
-                    progname, def_realm);
+                    progname, realm);
             return retval;
         }
     }
 
-    retval = krb5_sname_to_principal(kpropd_context, NULL, KIPROP_SVC_NAME,
-                                     KRB5_NT_SRV_HST, &iprop_svc_principal);
+    retval = sn2princ_realm(kpropd_context, NULL, KIPROP_SVC_NAME, realm,
+                            &iprop_svc_principal);
     if (retval) {
         com_err(progname, retval,
                 _("while trying to construct host service principal"));
         return retval;
     }
 
-    /* XXX referrals? */
-    if (krb5_is_referral_realm(krb5_princ_realm(kpropd_context,
-                                                iprop_svc_principal))) {
-        krb5_data *r = krb5_princ_realm(kpropd_context,
-                                        iprop_svc_principal);
-        assert(def_realm != NULL);
-        r->length = strlen(def_realm);
-        r->data = strdup(def_realm);
-        if (r->data == NULL) {
-            com_err(progname, retval,
-                    _("while determining local service principal name"));
-            return retval;
-        }
-        /* XXX Memory leak: Old r->data value.  */
-    }
     retval = krb5_unparse_name(kpropd_context, iprop_svc_principal,
                                &iprop_svc_princstr);
     if (retval) {
@@ -999,6 +972,8 @@ done:
     free(master_svc_princstr);
     krb5_free_default_realm(kpropd_context, def_realm);
     kadm5_destroy(server_handle);
+    krb5_db_fini(kpropd_context);
+    ulog_fini(kpropd_context);
     krb5_free_context(kpropd_context);
 
     return (runonce == 1) ? 0 : 1;
@@ -1157,21 +1132,29 @@ parse_args(char **argv)
     if (!debug)
         set_com_err_hook(kpropd_com_err_proc);
 
+    if (realm == NULL) {
+        retval = krb5_get_default_realm(kpropd_context, &def_realm);
+        if (retval) {
+            com_err(progname, retval, _("Unable to get default realm"));
+            exit(1);
+        }
+        realm = def_realm;
+    } else {
+        retval = krb5_set_default_realm(kpropd_context, realm);
+        if (retval) {
+            com_err(progname, retval, _("Unable to set default realm"));
+            exit(1);
+        }
+    }
+
     /* Construct service name from local hostname. */
-    retval = krb5_sname_to_principal(kpropd_context, NULL, KPROP_SERVICE_NAME,
-                                     KRB5_NT_SRV_HST, &server);
+    retval = sn2princ_realm(kpropd_context, NULL, KPROP_SERVICE_NAME, realm,
+                            &server);
     if (retval) {
         com_err(progname, retval,
                 _("while trying to construct my service name"));
         exit(1);
     }
-    if (realm != NULL) {
-        retval = krb5_set_principal_realm(kpropd_context, server, realm);
-        if (retval) {
-            com_err(progname, errno, _("while constructing my service realm"));
-            exit(1);
-        }
-    }
 
     /* Construct the name of the temporary file. */
     if (asprintf(&temp_file_name, "%s.temp", file) < 0) {
@@ -1180,6 +1163,8 @@ parse_args(char **argv)
         exit(1);
     }
 
+    params.realm = realm;
+    params.mask |= KADM5_CONFIG_REALM;
     retval = kadm5_get_config_params(kpropd_context, 1, &params, &params);
     if (retval) {
         com_err(progname, retval, _("while initializing"));
