@@ -319,6 +319,7 @@ main(int argc, char **argv)
     krb5_principal principal;
     krb5_creds creds;
     krb5_error_code err;
+    const char *errmsg;
     char *opt, *val;
     struct responder_data response;
     int c;
@@ -419,8 +420,10 @@ main(int argc, char **argv)
         fprintf(stderr, "error: responder callback wasn't called\n");
         err = 1;
     } else if (err) {
+        errmsg = krb5_get_error_message(context, err);
         fprintf(stderr, "error: krb5_get_init_creds_password failed: %s\n",
-                krb5_get_error_message(context, err));
+                errmsg);
+        krb5_free_error_message(context, errmsg);
         err = 2;
     }
     krb5_free_context(context);
